// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of scatter
//
// scatter is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// scatter is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the
//       Free Software Foundation, Inc.
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#include <algorithm>
#include <iterator>

#include  "ScatterDB.h"
#include  "EsaLinPowSymDB.h"
#include  "EsaLinAsymDB.h"


using namespace std;

namespace Scatter {

    double
    ScatterDB::interpolate( double x, double x1, double y1, double x2, double y2 ) {

	double y;
	if ( x1 == x2 )
	    y = y1;
	else {
	    double f2 = ( x - x1 ) / ( x2 - x1 );
	    double f1 = 1 - f2;
	    y  =  f1 * y1 + f2 * y2;
	}

	return y;
    }


    ScatterDB*
    ScatterDB::load( const string& filename ) {

	if ( filename.empty() )
	    return NULL;

	int nmatches =
	    EsaLinPowSymDB::db_matches( filename )
	    + EsaLinAsymDB::db_matches( filename );

	if ( nmatches > 1 )
	    throw( Exception ( "scatter db file " + filename + "matches more than one format??\n" ) );

	if ( EsaLinPowSymDB::db_matches( filename ) )
	    return new EsaLinPowSymDB( filename );

	else if ( EsaLinAsymDB::db_matches( filename ) )
	    return new EsaLinAsymDB( filename );


	else
	    throw( Exception( "scatter db file" + filename + "doesn't match any known formats\n" ) );
    }

    bool
    ScatterDB::find_bounds( ColData &data, double value,
			    pair<ColData::iterator, ColData::iterator>& range,
			    pair<long,long>& idx
			    ) {

	bool is_extrapolated;

	range = equal_range( data.begin(), data.end(), value );

	// first == second == begin() =>   esa < data[0]
	// first == second == end()   =>         data[n] < esa
	// first == second            =>   first - 1 < esa < second
	// first < second             =>   first == esa


	// first < second             =>   first == esa
	if ( distance(range.first, range.second) ) {

	    is_extrapolated = false;

	}

	// first == second == begin()   =>         data[n] < esa
	else if ( range.first == data.begin() ) {

	    is_extrapolated = true;

	}

	// first == second == end()   =>         data[n] < esa
	else if ( range.second == data.end() ) {

	    range.first = range.second = data.end() - 1;

	    is_extrapolated = true;

	}

	// first == second            =>   first - 1 < esa < second
	else {

	    --range.first;
	    is_extrapolated = false;

	}

	idx.first  = distance( data.begin(), range.first );
	idx.second = distance( data.begin(), range.second );

	return is_extrapolated;
    }


}
