#ifndef SCATTER_H
#define SCATTER_H

// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of scatter
//
// scatter is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// scatter is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the
//       Free Software Foundation, Inc.
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--


#include <bpipe/datatypes.h>
#include <mathconst/mathconst.h>

#include "ZoneMap.h"


#define ARCSEC2RAD(theta) ( (theta / 3600) * (M_PI / 180) )
#define RAD2ARCSEC(theta) ( (theta / M_PI) * 180 * 3600 )

namespace Scatter {

    enum ScatterStatus {
	OK                     = 0x0000,
	SCATTERED_INTO_SURFACE = 0x0001,
	BACKSCATTERED          = 0x0002,
	IP_EXTRAPOLATED        = 0x0004,
	OP_EXTRAPOLATED        = 0x0008
    };


}

unsigned int scatter(
    const Scatter::ZoneMap::Zone* zone,
    int iter_limit,
    double theta_def, double phi_def,
    double min_cum_prob, double max_cum_prob,
    const unsigned int id, const double energy,
    const DVector3& surface_norm, DVector3& direction,
    double& theta_o, double& phi_o
    );

#endif // #ifndef SCATTER_H
